@@ -33,7 +33,7 @@ import re
 import sys
 import cgi
 import urllib
-from _looper import looper
+from tempita._looper import looper
 
 __all__ = ['TemplateError', 'Template', 'sub', 'HTMLTemplate',
            'sub_html', 'html', 'bunch']
@@ -47,13 +47,13 @@ class TemplateError(Exception):
     """
 
     def __init__(self, message, position, name=None):
-        self.message = message
+        Exception.__init__(self, message)
         self.position = position
         self.name = name
 
     def __str__(self):
         msg = '%s at line %s column %s' % (
-            self.message, self.position[0], self.position[1])
+            ' '.join(self.args), self.position[0], self.position[1])
         if self.name:
             msg += ' in %s' % self.name
         return msg
@@ -74,9 +74,27 @@ class Template(object):
 
     default_encoding = 'utf8'
 
-    def __init__(self, content, name=None, namespace=None):
+    def __init__(self, content, name=None, namespace=None, stacklevel=None):
         self.content = content
         self._unicode = isinstance(content, unicode)
+        if name is None and stacklevel is not None:
+            try:
+                caller = sys._getframe(stacklevel)
+            except ValueError:
+                pass
+            else:
+                globals = caller.f_globals
+                lineno = caller.f_lineno
+                if '__file__' in globals:
+                    name = globals['__file__']
+                    if name.endswith('.pyc') or name.endswith('.pyo'):
+                        name = name[:-1]
+                elif '__name__' in globals:
+                    name = globals['__name__']
+                else:
+                    name = '<string>'
+                if lineno:
+                    name += ':%s' % lineno
         self.name = name
         self._parsed = parse(content, name=name)
         if namespace is None:
@@ -112,6 +130,7 @@ class Template(object):
                     % (args[0],))
             kw = args[0]
         ns = self.default_namespace.copy()
+        ns['__name__'] = self.name
         ns.update(self.namespace)
         ns.update(kw)
         result = self._interpret(ns)
@@ -205,7 +224,7 @@ class Template(object):
         except:
             exc_info = sys.exc_info()
             e = exc_info[1]
-            if getattr(e, 'args'):
+            if getattr(e, 'args', None):
                 arg0 = e.args[0]
             else:
                 arg0 = str(e)
@@ -281,7 +300,6 @@ def sub(content, **kw):
     name = kw.get('__name')
     tmpl = Template(content, name=name)
     return tmpl.substitute(kw)
-    return result
 
 def paste_script_template_renderer(content, vars, filename=None):
     tmpl = Template(content, name=filename)
@@ -328,11 +346,15 @@ class html(object):
         self.value = value
     def __str__(self):
         return self.value
+    def __html__(self):
+        return self.value
     def __repr__(self):
         return '<%s %r>' % (
             self.__class__.__name__, self.value)
 
-def html_quote(value):
+def html_quote(value, force=True):
+    if not force and hasattr(value, '__html__'):
+        return value.__html__()
     if value is None:
         return ''
     if not isinstance(value, basestring):
@@ -374,14 +396,20 @@ class HTMLTemplate(Template):
         html=html,
         attr=attr,
         url=url,
+        html_quote=html_quote,
         ))
 
     def _repr(self, value, pos):
-        plain = Template._repr(self, value, pos)
-        if isinstance(value, html):
-            return plain
+        if hasattr(value, '__html__'):
+            value = value.__html__()
+            quote = False
         else:
+            quote = True
+        plain = Template._repr(self, value, pos)
+        if quote:
             return html_quote(plain)
+        else:
+            return plain
 
 def sub_html(content, **kw):
     name = kw.get('__name')
@@ -733,8 +761,7 @@ def fill_command(args=None):
     options, args = parser.parse_args(args)
     if len(args) < 1:
         print 'You must give a template filename'
-        print dir(parser)
-        assert 0
+        sys.exit(2)
     template_name = args[0]
     args = args[1:]
     vars = {}
